<?php

namespace App\View\Helper;

use Cake\Core\Configure;
use Cake\View\Helper;
use Cake\View\StringTemplateTrait;
use Cake\I18n\I18n;

class ListsHelper extends Helper
{

    use StringTemplateTrait;

    protected $_defaultConfig = [
        'templates' => [
            'review' => '
                        <li {{reviewLi}}>
                            <div{{attrsReview}}>
                                <h3>{{title}}</h3>
                                <span>{{created}}</span>
                                {{stars}}
                                <span>{{userName}}</span>
                                <p>{{reviewComment}}</p>
                            </div>
                            
                        </li>'
                         ,
            'stars' => '<span{{attrGlyph}}><span{{attrStar}}>{{content}}</span></span>',
            'software' =>
                '<li {{attrsColumn}}>
                    <div {{attrsBlock}}>
                        <h3>{{softwareName}}</h3>
                        <div>
                            {{LinkImage}}
                        </div>
                        <p {{attrsDescription}}>{{softwareDescription}}</p>
                    </div>
                </li>',
            'user' => '<li {{attrsColumn}}>
                    <div {{attrsBlock}}>
                        <h3 {{attrsTitle}}>{{name}}</h3>
                        <div>
                            {{LinkImage}}
                        </div>
                    </div>
                </li>',
            'header' => '<div class="align"><h2>{{sectionTitle}}</h2><span class="allSee" {{attribs}}>{{link}}</span></div>',
            'footer' => '<div class="addmore">{{link}}</div>'
        ]
    ];

    public $helpers = ['Html', 'Text', 'Time'];

    public function softwares($softwares, $limit = false)
    {
        $result = null;
        if (!empty($softwares)) {
            foreach ($softwares as $software) {
                if (empty ($software->logo_directory) || empty($software->photo)) {
                    $logo = $this->Html->link($this->Html->image("logos/Softwarelogo_placeholder.jpg",
                        ["alt" => $software->softwarename, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Softwares', 'action' => 'view', $software->id],
                        ['escape' => false, "title" => $software->softwarename]);
                } else {
                    $logo = $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $software->logo_directory . DS . $software->photo,
                        ["alt" => $software->softwarename, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Softwares', 'action' => 'view', $software->id],
                        ['escape' => false, "title" => $software->softwarename]);
                }

                /**
                 * Define css and bootstrap according to home page (4 col) or other pages (6 col) for softwareBlock.
                 * //!\\ Bootstrap and customise Css Have to be on different html tags !
                 */
                if ($limit == false) {
                    $attrsColumn = $this->templater()->formatAttributes(['class' => "col-xs-12 col-sm-4 col-md-3 col-lg-2"]);
                    $attrsBlock = $this->templater()->formatAttributes(['class' => 'software-unit-all backgroundUnit']);
                } else {
                    $attrsColumn = $this->templater()->formatAttributes(['class' => 'col-xs-12 col-sm-6 col-md-3 col-lg-3']);
                    $attrsBlock = $this->templater()->formatAttributes(['class' => 'software-unit-home backgroundUnit']);
                }

                $result .= $this->formatTemplate('software', [
                    'attrsColumn' => $attrsColumn,
                    'attrsBlock' => $attrsBlock,
                    'softwareName' =>
                        $this->Html->link(
                            $this->Text->truncate($software->softwarename, 12, ['ellipsis' => '...', 'exact' => false]),
                            ['controller' => 'Softwares', 'action' => 'view', $software->id],
                            ['escape' => false]
                        ),
                    'LinkImage' => $logo,
                    'attrsDescription' => $this->templater()->formatAttributes(['class' => 'text-overflow project-description']),
                    'softwareDescription' => $this->Text->truncate($software->description, 100, ['ellipsis' => '...', 'exact' => false]),
                    ['class' => 'text-overflow project-description'],
                ]);
            }
        }
        return null !== $result ? $this->Html->tag('ul', $result, ['class' => 'row list-unstyled']) : null;
    }

    /**
     * @param $softwares should contain a list of software
     * @param array $options options for create a link and a specific message
     * @return string Returns a block of software's summary
     */
    public function softwareBlock($softwares, array $options, $limit = true)
    {
        $link = "";
        $softwareDisplay = "";
        if (!empty($softwares)) {

            if ((count($softwares) > Configure::read("MAX_DISPLAY")) && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' => $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $softwareDisplay = ($limit == true) ? $this->softwares(array_slice($softwares, 0, Configure::read("MAX_DISPLAY")), $limit) : $this->softwares($softwares, $limit);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle' => $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($softwareDisplay, '<li') == false) {
            $softwareDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );
        return $header . $softwareDisplay . $footer;
    }

    /**
     * @param $users
     * @param bool $limit : true -> 4 columns else 6 columns
     * @return null
     */
    public function users($users, $limit = false)
    {
        $result = null;

        if (!empty($users)) {
            foreach ($users as $user) {
                if ((empty ($user->logo_directory) || empty($user->photo))) {
                    $logo = $this->Html->link($this->Html->image("logos/User_placeholder.jpg",
                        ["alt" => $user->username, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Users', 'action' => 'view', $user->id],
                        ['escape' => false]);
                } else {
                    $logo = $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $user->logo_directory . DS . $user->photo,
                        ["alt" => $user->username, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Users', 'action' => 'view', $user->id],
                        ['escape' => false]);
                }

                /**
                 * Definite css and bootstrap according to home page (4 col) or other pages (6 col) for UsersBlock.
                 * //!\\ Bootstrap and customise Css Have to be on different html tags !
                 */
                if ($limit == false) {
                    $attrsColumn = $this->templater()->formatAttributes(['class' => "col-xs-12 col-sm-4 col-md-3 col-lg-2"]);
                    $attrsBlock = $this->templater()->formatAttributes(['class' => 'user-unit-all backgroundUnit list-unstyled']);
                } else {
                    $attrsColumn = $this->templater()->formatAttributes(['class' => 'col-xs-12 col-sm-6 col-md-3 col-lg-3']);
                    $attrsBlock = $this->templater()->formatAttributes(['class' => 'user-unit-home backgroundUnit list-unstyled']);
                }

                $result .= $this->formatTemplate('user', [
                    'attrsColumn' => $attrsColumn,
                    'attrsBlock' => $attrsBlock,
                    'name' =>
                        $this->Html->link($this->Text->truncate($user->username, 35, ['ellipsis' => '...', 'exact' => false]),
                            ['controller' => 'Users', 'action' => 'view', $user->id],
                            ['escape' => false, "title" => $user->username]
                        ),
                    'attrsTitle' => $this->templater()->formatAttributes(['class' => 'size-title']),
                    'LinkImage' => $logo,
                ]);
            }
        }
        return null !== $result ? $this->Html->tag('ul', $result, ['class' => 'row list-unstyled']) : null;
    }

    /**
     * @param $users Should be contains a list of users
     * @param array $options options for create a link and a specific message
     * @return string Returns a block of software's summary
     */
    public function userBlock($users, array $options, $limit = true)
    {
        $link = "";
        $userDisplay = "";
        if (!empty($users)) {

            if ((count($users) > Configure::read("MAX_DISPLAY")) && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' => $options["titleSeeAll"]],
                    ['escape' => false]);
            }
            $userDisplay = ($limit == true) ? $this->users(array_slice($users, 0, Configure::read("MAX_DISPLAY")), $limit) : $this->users($users, $limit);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle' => $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($userDisplay, '<li') == false) {
            $userDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );
        return $header . $userDisplay . $footer;
    }

    /**
     * @param $screenshots
     * @return null
     */
    public function screenshots($screenshots)
    {
        $result = null;
        if (!empty($screenshots)) {
            foreach ($screenshots as $screenshot) {
                if (!empty ($screenshot->url_directory) || !empty($screenshot->photo)) {
                    $result .= $this->Html->tag(
                        'li',
                        $this->Html->tag(
                            'div',
                            $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $screenshot->url_directory . DS . $screenshot->photo,
                                ['alt' => __d("Screenshot of software {0}", ""), 'class' => 'img-responsive ']),
                                COMPTOIR_SRV_URL . $screenshot->url_directory . DS . $screenshot->photo,
                                ['escape' => false])
                            ,
                            ['class' => 'size-title screenShot']
                        )
                        ,
                        ["class" => " col-xs-12 col-sm-6 col-md-3 col-lg-3 "]
                    );
                }
            }
        }

        return null !== $result ? $this->Html->tag('ul', $result, ['class' => 'row list-unstyled']): null;

    }

    public function screenshotBlock($items, array $options, $limit = true)
    {
        $link = "";
        $screenshotDisplay = "";
        if (!empty($items)) {

            if ((count($items) > Configure::read("MAX_DISPLAY")) && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' => $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $screenshotDisplay = ($limit == true) ? $this->screenshots(array_slice($items, 0, Configure::read("MAX_DISPLAY"))) : $this->screenshots($items);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle' => $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($screenshotDisplay, '<li') == false) {
            $screenshotDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );
        return $header . $screenshotDisplay . $footer;
    }

    public function reviews($reviews)
    {
        $result = "";
        if (!empty($reviews)) {
            foreach ($reviews as $review) {
                $starsElement = "";
                for ($stars = 0; $stars < $review->evaluation; $stars++) {
                    $starsElement .= $this->formatTemplate('stars',
                        [
                            'attrGlyph' => $this->templater()->formatAttributes(["class" => "glyphicon glyphicon-star"]),
                            'attrStar' => $this->templater()->formatAttributes(["class" => "sr-only"]),
                            'content' => "*",
                        ]
                    );
                }

                $result .= $this->formatTemplate('review', [
                    'reviewLi' => $this->templater()->formatAttributes(['class' => "col-xs-11 col-sm-6 col-md-6 col-lg-6"]),
                    'attrsReview'=> $this->templater()->formatAttributes(['class' => "backgroundReview"]),
                    'title' => $this->Text->truncate($review->title, 100, ['ellipsis' => '...', 'exact' => false]),
                    'userName' => $this->Html->link($this->Text->truncate($review->user->username, 35, ['ellipsis' => '...', 'exact' => false]),
                        ['controller' => 'Users', 'action' => 'view', $review->user->id],
                        ['escape' => false, "title" => $review->user->username]),
                    'created' => $this->Time->format(strtotime($review->created),
                        [\IntlDateFormatter::SHORT, -1],
                        I18n::locale()
                    ),
                    'starDiv' => $this->templater()->formatAttributes(['class' => 'ratingStar']),
                    'stars' => $starsElement . $review->evaluation,
                    'reviewComment' => $this->Text->truncate($review->comment, 350, ['ellipsis' => '...', 'exact' => false]),
                ]);
            }
        }
        return null !== $result ? $this->Html->tag('ol', $result, ['class' => 'row list-unstyled']): null;
    }

    public function reviewBlock($items, array $options, $limit = true)
    {
        $link = "";
        $reviewDisplay = "";
        if (!empty($items)) {

            if (count($items) > Configure::read("MAX_DISPLAY") && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' => $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $reviewDisplay = ($limit == true) ? $this->reviews(array_slice($items, 0, Configure::read("MAX_DISPLAY"))) : $this->reviews($items);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle' => $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($reviewDisplay, '<li') == false) {
            $reviewDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );
        return $header . $reviewDisplay . $footer;
    }
}
